package simulator.workload.reader.archive.swf;

import org.qcg.broker.schemas.jobdesc.AppProperty;
import org.qcg.broker.schemas.jobdesc.Application;
import org.qcg.broker.schemas.jobdesc.ComputingResourceBaseTypeChoice;
import org.qcg.broker.schemas.jobdesc.ComputingResourceType;
import org.qcg.broker.schemas.jobdesc.ComputingResourceBaseTypeChoiceItem;
import org.qcg.broker.schemas.jobdesc.ComputingResourceParameterType;
import org.qcg.broker.schemas.jobdesc.Executable;
import org.qcg.broker.schemas.jobdesc.ExecutionTimeType;
import org.qcg.broker.schemas.jobdesc.ExecutionType;
import org.qcg.broker.schemas.jobdesc.ParameterTypeChoice;
import org.qcg.broker.schemas.jobdesc.ParameterTypeChoiceItem;
import org.qcg.broker.schemas.jobdesc.ParentType;
import org.qcg.broker.schemas.jobdesc.ResourceRequirementsType;
import org.qcg.broker.schemas.jobdesc.Value;
import org.qcg.broker.schemas.jobdesc.RequirementsType;
import org.qcg.broker.schemas.jobdesc.Workflow;
import org.qcg.broker.schemas.jobdesc.types.ComputingParameterName;
import org.qcg.broker.schemas.jobdesc.Task;
import org.qcg.broker.schemas.jobdesc.types.TaskStatesName;

import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.exolab.castor.types.Duration;

import simulator.workload.reader.archive.QcgWAJobReader;
import test.workloadandapp.AppTypeGenerator;



/**
 * 
 * @author Marcin Krystek
 *
 */
public class QcgSWFJobReader extends QcgWAJobReader {

	private Log log = LogFactory.getLog(QcgSWFJobReader.class);
	
	private static String DEFAULT_PATH_TO_APP_PROFILES = "";
	
	protected String currntJobID;
	
	protected AppTypeGenerator appTypeGen;
	
	public QcgSWFJobReader(String fileName) throws NullPointerException,
			IOException {
		super(fileName);
		this.waParser.loadHeader();
		this.appTypeGen = new AppTypeGenerator();
	}
	
	public QcgSWFJobReader(SWFParser parser) {
		super(parser);
	}

	protected Task createTask(String[] data) {
		RequirementsType requirements = null;
		ResourceRequirementsType taskReq = null;
		ComputingResourceType compResource = null;
		ComputingResourceBaseTypeChoice choice = null;
		ComputingResourceBaseTypeChoiceItem crItem = null;		
		ComputingResourceParameterType param = null;
		
		Task task = null;
		long value = -1;
		
		task = new Task();
		requirements = new RequirementsType();
		taskReq = new ResourceRequirementsType();
		compResource = new ComputingResourceType();
		choice = new ComputingResourceBaseTypeChoice();
		
		compResource.setComputingResourceBaseTypeChoice(choice);
		taskReq.addComputingResource(compResource);
		requirements.setResourceRequirements(taskReq);
		task.setRequirements(requirements);
		
		for(int i = 0; i < SWFFields.DATA_FIELDS.length; i++){
			try {
				
				value = Long.parseLong(data[SWFFields.DATA_FIELDS[i]]);
				
				switch(SWFFields.DATA_FIELDS[i]){
					case SWFFields.DATA_NUMBER_OF_ALLOCATED_PROCESSORS:
						if(value <= 0){
							if(log.isWarnEnabled())
								log.warn("Task "+data[0] + " is omitted. Number of allocated processors should be greater than 0.");
							return null;
						}
						break;
					case SWFFields.DATA_REQUESTED_NUMBER_OF_PROCESSORS:
						if(value <= 0){
							if(log.isWarnEnabled())
								log.warn("Task "+data[0] + " is omitted. Number of requested processors should be greater than 0.");
							return null;
						}
						break;
					case SWFFields.DATA_RUN_TIME:
						if(value <= 0){
							if(log.isWarnEnabled())
								log.warn("Task "+data[0] + " is omitted. Task runtime should be greater than 0.");
							return null;
						}
						break;
				}
				
				if (value == -1 && i != SWFFields.DATA_EXECUTABLE_NUMBER) continue;
				
			} catch (NumberFormatException e) {
				if(SWFFields.DATA_FIELDS[i] != SWFFields.DATA_JOB_NUMBER &&
					SWFFields.DATA_FIELDS[i] != SWFFields.DATA_PRECEDING_JOB_NUMBER)
					continue;
			}

			switch(SWFFields.DATA_FIELDS[i]){
				case SWFFields.DATA_JOB_NUMBER:
					currntJobID = data[SWFFields.DATA_FIELDS[i]];
					String tab[] = this.waParser.getIDMapping(currntJobID);
					task.setId(tab[1]);
					break;
					
				case SWFFields.DATA_SUBMIT_TIME:
					/*  
					 * This is information for simulator not for task description.
					 * See WorkloadLoader for details.
					 */ 
					break;
					
				case SWFFields.DATA_WAIT_TIME:
					break;
					
				case SWFFields.DATA_RUN_TIME:
					/*  
					 * This is information for simulator not for task description.
					 * See WorkloadLoader for details.
					 */
					break;
					
				case SWFFields.DATA_NUMBER_OF_ALLOCATED_PROCESSORS:
					/*  
					 * This is information for simulator not for task description.
					 * See WorkloadLoader for details.
					 */
					break;
					
				case SWFFields.DATA_AVERAGE_CPU_TIME_USED:
					break;
					
				case SWFFields.DATA_USED_MEMORY:
					/*  
					 * This is information for simulator not for task description.
					 * See WorkloadLoader for details.
					 */
					break;
					
				case SWFFields.DATA_REQUESTED_NUMBER_OF_PROCESSORS:
					
					crItem = new ComputingResourceBaseTypeChoiceItem();
					param = new ComputingResourceParameterType();
					
					param.setName(ComputingParameterName.CPUCOUNT);
					{
						ParameterTypeChoice tpChoice = new ParameterTypeChoice();
						ParameterTypeChoiceItem item = new ParameterTypeChoiceItem();
						Value paramValue = new Value();
						
						paramValue.setContent(Long.valueOf(value).doubleValue());
						item.setValue(paramValue);
						tpChoice.addParameterTypeChoiceItem(item);
						param.setParameterTypeChoice(tpChoice);
					}
					crItem.setHostParameter(param);
					choice.addComputingResourceBaseTypeChoiceItem(crItem);
					
					break;
					
				case SWFFields.DATA_REQUESTED_TIME:
					ExecutionTimeType executionTime = new ExecutionTimeType();

					// requested time is expressed in swf file in seconds
					// multiply value * 1000 to get milliseconds
					Duration d = new Duration(value * 1000);
					
					executionTime.setExecutionDuration(d);
					task.setExecutionTime(executionTime);
					break;
					
				case SWFFields.DATA_REQUESTED_MEMORY:
					crItem = new ComputingResourceBaseTypeChoiceItem();
					param = new ComputingResourceParameterType();
					
					param.setName(ComputingParameterName.MEMORY);
					{
						ParameterTypeChoice tpChoice = new ParameterTypeChoice();
						ParameterTypeChoiceItem item = new ParameterTypeChoiceItem();
						Value paramValue = new Value();
						
						paramValue.setContent(Long.valueOf(value).doubleValue());
						item.setValue(paramValue);
						tpChoice.addParameterTypeChoiceItem(item);
						param.setParameterTypeChoice(tpChoice);
					}
					
					crItem.setHostParameter(param);
					choice.addComputingResourceBaseTypeChoiceItem(crItem);
					
					break;
					
				case SWFFields.DATA_STATUS:
					break;
					
				case SWFFields.DATA_USER_ID:
					break;
					
				case SWFFields.DATA_GROUP_ID:
					break;
					
				case SWFFields.DATA_EXECUTABLE_NUMBER:
					String pathToAppProfile = waParser.getAppMapping().get(String.valueOf(value));
					String appName = null;
					if(pathToAppProfile == null){
						if(value != -1){
							appName = String.valueOf(value);
						} else {
							appName = appTypeGen.randomAppType();
						}
						pathToAppProfile =  DEFAULT_PATH_TO_APP_PROFILES + appName;
					} else {
						String fullAppName = pathToAppProfile.substring(pathToAppProfile.lastIndexOf('/') + 1, pathToAppProfile.length());
						if(fullAppName != null)
							appName = fullAppName.substring(0, fullAppName.lastIndexOf('.'));
						else appName = fullAppName;
					}
					ExecutionType execType = new ExecutionType();
					Executable executable = new Executable();
					Application application = new Application();
					AppProperty appProp = new AppProperty();
					appProp.setName("appId");
					appProp.setContent(String.valueOf(value));
					application.addAppProperty(appProp);
					/*appProp = new AppProperty();
					appProp.setName("appProfileLocation");
					appProp.setContent(pathToAppProfile);
					application.addAppProperty(appProp);*/
					application.setName(appName);
					executable.setApplication(application);
					execType.setExecutable(executable);
					task.setExecution(execType);
					break;
					
				case SWFFields.DATA_QUEUE_NUMBER:
					break;
					
				case SWFFields.DATA_PARTITION_NUMBER:
					break;
					
				case SWFFields.DATA_PRECEDING_JOB_NUMBER:
					/*Workflow workflow = new Workflow();
					ParentType parent = new ParentType();
					parent.setTriggerState(TaskStatesName.FINISHED);
					workflow.addParent(parent);
					task.setWorkflow(workflow);*/
					break;
					
				case SWFFields.DATA_THINK_TIME_FROM_PRECEDING_JOB:
					break;
			}
		}
		
		return task;
	}
	
}
